<?php
/**
 * RegistroCCAGED
 *
 * @package    Logic
 * @subpackage Registro/Caged
 *
 * @author     brunofeliciano - 02/06/2011
 *
 */
class RegistroCCAGED extends RegistroCAGED{

    const REGISTRO_C = 'C';
    const SEQUENCIA = '00000';

    private $_tpInscricao;
    private $_nrInscricao;
    private $_pessoa;
    private $_vinculo;


    /**
     * Gera Tipo de Registro C.
     *
     * @return void
     *
     * @author thalesvalias - 23/05/2012
     */
    public function geraRegistroC(){
        // Verifica se o vínculo possui PIS/PASEP cadastrado, caso contrário o mesmo não será gerado no arquivo. leonardocarvalho 05/10/2013
        if ((bool)$this->_pessoa->historico->getNrpispaseppes()) {
            $this->_geraRegistro();
        } else {
            tk_warning ("Vínculo " . $this->_vinculo->getNrvinculom(). " não foi gerado pois PIS/PASEP não foi informado.");
        }
    }


    /**
     * Atribui os dados no atributo da classe Pai.
     *
     * @return void
     *
     * @author thalesvalias - 23/05/2012 / leonardocarvalho
     */
    private function _geraRegistro(){

        // Bloco 1 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_C, null, null);

        // Bloco 2 | 002 | Tipo de Identificacao.
        $this->atribuiDado($this->_tpInscricao, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 3 | 003-016 | Numero do Identificador do Estabelecimento.
        $this->atribuiDado($this->_nrInscricao, 14, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Sequencia criada ao gerar o arquivo CAGED.txt.
        // Bloco 4 | 017-021 | Sequencia.
        $this->atribuiDado(self::SEQUENCIA, 5);

        // Bloco 5 | 022-032 | PIS/PASEP.
        $this->_verificaPis($this->_pessoa->historico->getNrpispaseppes()); // Verifica se é PIS/PASEP conforme regra do CAGED
        $this->atribuiDado($this->_pessoa->historico->getNrpispaseppes(), 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 6 | 033 | Sexo (1-Masculino / 2-Feminino).
        $this->_atribuiSexo($this->_pessoa->historico->getIdsexopessoa());

        // Bloco 7 | 034-041 | Nascimento (DDMMAAAA).
        $this->atribuiDado($this->_pessoa->historico->getDtnascpessoa(), 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 8 | 042-043 | Instrucao.
        // 1 - Analfabeto, inclusive os que embora tenham recebido instruaoo, nao se alfabetizaram ou tenham esquecido.
        // 2 - Ate quarta serie incompleta do primeiro grau (primario incompleto ou que se tenha alfabetizado sem ter frequentado
        // escola regular).
        // 3 - Quarta serie completa do primeiro grau (primario completo).
        // 4 - Da quinta a oitava serie incompleta do primeiro grau (primario completo).
        // 5 - primeiro grau (ginasial) completo.
        // 6 - segundo grau (colegial) incompleto.
        // 7 - segundo grau (colegial) completo.
        // 8 - Superior incompleto.
        // 9 - Superior completo.
        if ((bool)$this->_pessoa->instrucao) {
            $this->atribuiDado($this->_pessoa->instrucao, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        } else {
            $this->atribuiDado($this->_pessoa->instrucao, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            tk_warning ("Grau de instrução não informado para o vínculo " . $this->_vinculo->getNrvinculom());
        }

        // Bloco 9 | 044-047 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 4, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 10 | 048-055 | Remuneracao.
        $this->atribuiDado($this->_vinculo->salario, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 11 | 056-057 | Horas Trabalhadas (de 01 a 44).
        $this->atribuiDado($this->_vinculo->escalaSemanal, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 12 | 058-065 | Admissao (DDMMAAAA).
        $this->atribuiDado($this->_vinculo->getDtadmissaovinc(), 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 13 | 066-067 | Tipo de Movimento.
        // ADMISSAO
        // 10 - Primeiro emprego.
        // 20 - Reemprego.
        // 25 - Contrato por prazo determinado.
        // 35 - Reintegracao.
        // 70 - Transferencia de entrada.
        // DESLIGAMENTO
        // 31 - Dispensa sem justa causa.
        // 32 - Dispensa por justa causa.
        // 40 - A pedido (espontaneo).
        // 43 - Termino de contrato por prazo determinado.
        // 45 - Termino de contrato.
        // 50 - Aposentado.
        // 60 - Morte.
        // 80 - Transferencia de saida.
        if ($this->_vinculo->tipoMovimentacao !== null){
            $this->atribuiDado($this->_vinculo->tipoMovimentacao, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        } else {
            $nrtpmovimento = 10; // Definido para gerar o CAGED.
            $this->atribuiDado($nrtpmovimento, 2, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            tk_warning('Transferência do movimento não informada para o vínculo '.$this->_vinculo->getNrvinculom());
        }

        // Bloco 14 | 068-069 | Dia de Desligamento.
        // Demissao - dia de saida.
        $this->_atribuiDataRescisao($this->_vinculo->getDtrescisaovinc());

        // Bloco 15 | 070-109 | Nome do Empregado.
        $this->atribuiDado($this->_pessoa->historico->getNmpessoa(), 40, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 16 | 110-117 | Numero da Carteira de Trabalho.
        $this->atribuiDado($this->_pessoa->historico->getNrctpspessoa(), 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 17 | 118-121 | Serie da Carteira de Trabalho.
        $this->atribuiDado($this->_pessoa->historico->getNrseriectpspes(), 4, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA, 1);

        // Bloco 18 | 122-128 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 7, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 19 | raca cor.
        //1 - Indígena
        //2 - Branca
        //4 - Preta
        //6 - Amarela
        //8 - Parda
        //9 - Não informado
        if ($this->_pessoa->raca !== null) {
            $this->atribuiDado($this->_pessoa->raca, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        } else {
            $nrraca = 9; // Definido para vínculos sem raça informada
            $this->atribuiDado($nrraca, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            tk_warning('Raça não informada para o vínculo '.$this->_vinculo->getNrvinculom());
        }

        // Bloco 20 | 130 | Portador de Deficiencia (1-sim / 2-nao).
        $this->atribuiDado($this->_pessoa->portadorDeficiencia, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 21 | 131-136 | CBO.
        $this->atribuiDado($this->_vinculo->cbo, 6, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 22 | 137 | Aprendiz (1-sim / 2-nao).
        $this->atribuiDado($this->_vinculo->empregaticio, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 23 | 138-139 | UF da Carteira de Trabalho (CTPS).
        if ((bool)$this->_pessoa->historico->getSgufctpspes()) {
            $this->atribuiDado($this->_pessoa->historico->getSgufctpspes(), 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
        } else {
            $this->atribuiDado($this->_pessoa->historico->getSgufctpspes(), 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
            tk_warning("UF da carteira de trabalho não informado para o vínculo " . $this->_vinculo->getNrvinculom());
        }

        // Bloco 24 | 140 | Tipo da deficiencia fisica.
        //1. Fisica
        //2. Auditiva
        //3. Visual
        //4. Mental
        //5. Multipla
        //6. Reabilitado
        $this->atribuiDado($this->_pessoa->condicaoFisica, 1, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

        // Bloco 25 | 141-151 | CPF.
        if ((bool)$this->_pessoa->historico->getNrcpfpessoa()) {
            $this->atribuiDado($this->_pessoa->historico->getNrcpfpessoa(), 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        } else {
            $this->atribuiDado($this->_pessoa->historico->getNrcpfpessoa(), 11, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
            tk_warning("CPF não informado para o vínculo " . $this->_vinculo->getNrvinculom());
        }

        // Bloco 26 | 152-159 | CEP Residencia do Trabalhador.
        $this->atribuiDado($this->_pessoa->cep, 8, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 27 | 160-240 | Filler.
        $this->atribuiDado(parent::CARACTER_BRANCO, 81, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);

    }


    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }


    public function setPessoa($pessoa)
    {
        $this->_pessoa = $pessoa;
    }


    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }


    public function setVinculo($vinculo)
    {
        $this->_vinculo = $vinculo;
    }


    /**
     * Converte o valor do sexo para 1 ou 2 (1 = M, 2 = F).
     *
     * @param string $sexo Sexo da pessoa.
     *
     * @return void
     *
     * @author thalesvalias 13/06/2012
     */
    private function _atribuiSexo($sexo){

        if ($sexo == GpepessoahEntity::IDSEXOPESSOA_MASCULINO) {
            $sexo = 1;
        } else if ($sexo == GpepessoahEntity::IDSEXOPESSOA_FEMININO) {
            $sexo = 2;
        }

        $this->atribuiDado($sexo, 1, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


 /**
 * Seta o dia da rescisao do vinculo.
 *
 * @param string $dtRescisao Data de rescisao do vinculo.
 *
 * @return void
 *
 * @author thalesvalias 24/05/2012
 */
    private function _atribuiDataRescisao($dtRescisao){

        $diaRescisao = null;
        if (empty($dtRescisao) === false) {
            $mesCompetencia = \FPA::getDtCompetencia();
            $ultimodiaCompetencia = \FPA::getUltimoDiaMes();
            $dtRescisao = new TkDateTime($dtRescisao);

            if ($dtRescisao->isBetween($mesCompetencia, $ultimodiaCompetencia)) {

                $dataRescisao = new TkDateTime($dtRescisao);
                $diaRescisao = $dataRescisao->toString(TkDateTime::DAY);

            }
        }

        $this->atribuiDado($diaRescisao, 2, parent::CARACTER_BRANCO, parent::PREENCHE_DIREITA);
    }

    /**
     * Verifica se é PIS/PASEP de acordo com a regra do CAGED.
     *
     * @param string $pis Pis do vinculo.
     *
     * @return void
     *
     * @author leonardocarvalho 04/09/2013
     */
    private function _verificaPis($pis){

        $range = array("109","110","111","112","113","114","115","116","117","118","119");
        $inicioPis = substr($pis, 0, 3);

        if (in_array($inicioPis, $range)) {
            tk_warning('O PIS iniciado de 109 a 119 não é PIS/PASEP, é número de Contribuinte Individual, vínculo '.$this->_vinculo->getNrvinculom());
        }

    }

}